/* ljprint - LaserJet driver; Copyright (c) 1996 by Andrew Maltsev <am@f1.ru>
 *
 * Based on JetPak public domain sources.
 *
 * See README & LICENSE files
 * New version may be obtained via URL:
 *  ftp://ftp.f1.ru/pub/am/ljprint/
*/
#define Version "1.0"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "patchlev.h"
#include "jetfont.h"
#include "jetutil.h"

static char pb[BUFSIZ];
static FONT_COMMAND fc;
#define FONTNUMBER 11459

static char *selection_sequence()
{ static char retstr[100];
  if (fc.data.font.spacing == PROPORTIONAL)
   { sprintf(retstr, "\033&l%dO\033(%d%c\033(s%du%dp%.1fv%ds%db%dt%dQ",
             fc.data.font.orientation,
             fc.data.font.symbol_set/32,
             '@'+fc.data.font.symbol_set%32,
             fc.data.font.placement,
             fc.data.font.spacing,
             quarter_dots_to_points(fc.data.font.height),
             fc.data.font.style,
             fc.data.font.stroke_weight,
             fc.data.font.typeface,
             fc.data.font.quality);
   }
  else
   { sprintf(retstr, "\033&l%dO\033(%d%c\033(s%du%dp%.1fh%.1fv%ds%db%dt%dQ",
             fc.data.font.orientation,
             fc.data.font.symbol_set/32,
             '@'+fc.data.font.symbol_set%32,
             fc.data.font.placement,
             fc.data.font.spacing,
             quarter_dots_to_lpi(fc.data.font.pitch),
             quarter_dots_to_points(fc.data.font.height),
             fc.data.font.style,
             fc.data.font.stroke_weight,
             fc.data.font.typeface,
             fc.data.font.quality);
   }
  return(retstr);
}

static
void
loadfont(const char *font, int load)
{ FILE *infp;
  int r;

  if(!(infp=fopen(font,"rb")))
   { fprintf(stderr, "Can't open %s\n", font);
     return;
   }

  /* just read the header information */
  while((r=font_command_read(infp, &fc))==OK &&
        fc.command_type!=FDC);
  if(r!=OK)
   { /* font header not found - probably not a soft font file */
     fprintf(stderr, "Can't read font header from %s (bad font file?)\n",font);
     fclose(infp);
     return;
   }

  /* reset printer, set CR and LF to be interpreted as is */
  sprintf(pb, "\033E\033&k0G");
  os_printstr(pb);

  if(load)	/* If we called to download the font */
   { /* download the font as temporary and secondary */
     sprintf(pb, "\033*c%dD\033*c4F", FONTNUMBER);
     os_printstr(pb);
     rewind(infp);
     while ((r = fread(pb, 1, BUFSIZ, infp)) > 0)
      os_printbuf(pb, r);
   }
  fclose(infp);
}

static
char table[]={0xed, 0xee, 0xf3, 0xf4, 0xf5, 0xf6, 0xf1, 0xf2, /* Graph */
              0xef, 0xf0, 0xec, 0x2a, 0x2a, 0x2a, 0x2a, 0x2a,
              0x2a, 0x2a, 0x2a, 0xee, 0x2a, 0x2a, 0x76, 0x20,
              0x20, 0x20, 0x20, 0xee, 0xeb, 0xe9, 0x2a, 0x3a,
              0xed, 0xee, 0xf3, 0xf8, 0xf3, 0xf3, 0xf4, 0xf4,
              0xf4, 0xf5, 0xf5, 0xf5, 0xf6, 0xf6, 0xf6, 0xf1,
              0xf1, 0xf1, 0xf2, 0xf7, 0xf2, 0xf2, 0xef, 0xef,
              0xef, 0xf0, 0xf0, 0xf0, 0xec, 0xec, 0xec, 0x20,
              0xde, 0xc0, 0xc1, 0xd6, 0xc4, 0xc5, 0xd4, 0xc3, /* Lower */
              0xd5, 0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce,
              0xcf, 0xdf, 0xd0, 0xd1, 0xd2, 0xd3, 0xc6, 0xc2,
              0xdc, 0xdb, 0xc7, 0xd8, 0xdd, 0xd9, 0xd7, 0xda,
              0xbe, 0xa0, 0xa1, 0xb6, 0xa4, 0xa5, 0xb4, 0xa3, /* Upper */
              0xb5, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae,
              0xaf, 0xbf, 0xb0, 0xb1, 0xb2, 0xb3, 0xa6, 0xa2,
              0xbc, 0xbb, 0xa7, 0xb8, 0xbd, 0xb9, 0xb7, 0xba};

static
void
printfile(const char *file, int conv)
{ FILE *f;
  int ch;

  f=fopen(file,"r");
  if(!f)
   { fprintf(stderr,"Can't read %s\n",file);
     return;
   }
  while((ch=fgetc(f))!=EOF)
   if(conv && ch>=128)
    os_printbuf(table+ch-128,1);
   else
    { char c;
      if(ch=='\n' || ch=='\f')
       c='\r',os_printbuf(&c,1);
      c=ch;
      os_printbuf(&c,1);
    } 
  fclose(f);
}

static
void
usage()
{ fprintf(stderr,
"ljprint " Version " Copyright (c) 1996 by Andrew Maltsev <am@f1.ru>\n"
"\n"
"Usage: ljprint [-o file] [-f fontfile] [-F] [-R] [-C]\n"
"               [-L] [-l left] [-t top] [-d lpi] [-c number] textfile ..\n"
"\t-o\twrite results to file (stdout by default)\n"
"\t-f\tuse font from fontfile\n"
"\t-F\tdo not load font into printer, just calculate escape\n"
"\t-R\tdon't reset printer after all\n"
"\t-C\tdo koi8-r->something character conversion (needed for cyrillic\n"
"\t\tfonts from ljiii.zip font pak)\n"
"\t-L\tswitch printer to landscape mode\n"
"\t-l\tset left margin\n"
"\t-t\tset top margin\n"
"\t-d\tset line-per-inch printer parameter\n"
"\t-c\tprint given nuber of copies of each page\n"
         );
  exit(1);
}

int
main(int ac, char **av)
{ int ch;
  char *fontfile=NULL;
  char *outfile=NULL;
  FILE *out;
  int ldfont=1;
  int left=-1,top=-1,lpi=-1,copies=-1;
  int landscape=0;
  int reset=1,convert=0;

  if(ac==1)
   usage();
  while((ch=getopt(ac,av,"Hf:FLl:t:d:o:Rc:C"))!=-1)
   switch(ch)
    { case 'f':
       fontfile=optarg;
       break;
      case 'F':
       ldfont=0;
       break;
      case 'C':
       convert=1;
       break;
      case 'L':
       landscape=1;
       break;
      case 'l':
       left=atoi(optarg);
       break;
      case 't':
       top=atoi(optarg);
       break;
      case 'd':
       lpi=atoi(optarg);
       break;
      case 'o':
       outfile=optarg;
       break;
      case 'R':
       reset=0;
       break;
      case 'c':
       copies=atoi(optarg);
       break;
      default:
       fputc('\n',stderr);
      case 'H':
      case '?':
       usage();
    }
  ac-=optind;
  av+=optind;

  if(outfile)
   { out=fopen(outfile,"w");
     if(!out)
      fprintf(stderr,"Can't write %s!\n",outfile),exit(1);
   }
  else
   { out=stdout;
     outfile="<stdout>";
   }
  fprintf(stderr,"Writing to %s..\n",outfile);
  os_setfile(out);

  if(fontfile)
   { fprintf(stderr,"%s font from file %s..\n",ldfont ? "Loading" : "Checking",fontfile);
     loadfont(fontfile,ldfont);
     os_printstr(selection_sequence());
   }

  if(landscape)
   { fprintf(stderr,"Landscape mode..\n");
     os_printstr("\033&l1O");
   }
  if(top>=0)
   { fprintf(stderr,"Top margin set to %d..\n",top);
     sprintf(pb,"\033&l%dE",top);
     os_printstr(pb);
   }
  if(left>=0)
   { fprintf(stderr,"Left margin set to %d..\n",left);
     sprintf(pb,"\033&a%dL",left);
     os_printstr(pb);
   }
  if(lpi>=0)
   { fprintf(stderr,"LPI set to %d..\n",lpi);
     sprintf(pb,"\033&l%dD",lpi);
     os_printstr(pb);
   }
  if(copies>0)
   { fprintf(stderr,"Each page will be printed %d times..\n",copies);
     sprintf(pb,"\033&l%dX",copies);
     os_printstr(pb);
   }

  for(; ac; ac--, av++)
   { fprintf(stderr,"Printing %s..\n",*av);
     printfile(*av,convert);
   }

  /* reset printer */
  if(reset)
   os_printstr("\033E");

  return 0;
}
